@@ -10,10 +10,7 @@
 """
 from jinja.filters import FILTERS as DEFAULT_FILTERS
 from jinja.tests import TESTS as DEFAULT_TESTS
-from jinja.utils import debug_context, safe_range
+from jinja.utils import NAMESPACE as DEFAULT_NAMESPACE
 
 
-DEFAULT_NAMESPACE = {
-    'range':                safe_range,
-    'debug':                debug_context
-}
+__all__ = ['DEFAULT_FILTERS', 'DEFAULT_TESTS', 'DEFAULT_NAMESPACE']
