@@ -8,26 +8,17 @@
     :copyright: 2007 by Armin Ronacher.
     :license: BSD, see LICENSE for more details.
 """
+import re
 from random import choice
+from operator import itemgetter
 from urllib import urlencode, quote
-from jinja.utils import escape, urlize
-from jinja.datastructure import Undefined
-from jinja.exceptions import FilterArgumentError
+from jinja.utils import urlize, escape, reversed, sorted, groupby, \
+     get_attribute, pformat
+from jinja.datastructure import TemplateData
+from jinja.exceptions import FilterArgumentError, SecurityException
 
 
-try:
-    _reversed = reversed
-except NameError:
-    # python2.3 compatibility hack for the do_reverse function
-    def _reversed(seq):
-        try:
-            return seq[::-1]
-        except:
-            try:
-                return list(seq)[::-1]
-            except:
-                raise TypeError('argument to _reversed must '
-                                'be a sequence')
+_striptags_re = re.compile(r'(<!--.*?-->|<[^>]*>)')
 
 
 def stringfilter(f):
@@ -50,6 +41,25 @@ def stringfilter(f):
     return decorator
 
 
+def simplefilter(f):
+    """
+    Decorator for simplifying filters. Filter arguments are passed
+    to the decorated function without environment and context. The
+    source value is the first argument. (like stringfilter but
+    without unicode conversion)
+    """
+    def decorator(*args):
+        def wrapped(env, context, value):
+            return f(value, *args)
+        return wrapped
+    try:
+        decorator.__doc__ = f.__doc__
+        decorator.__name__ = f.__name__
+    except:
+        pass
+    return decorator
+
+
 def do_replace(s, old, new, count=None):
     """
     Return a copy of the value with all occurrences of a substring
@@ -68,14 +78,14 @@ def do_replace(s, old, new, count=None):
     """
     if not isinstance(old, basestring) or \
        not isinstance(new, basestring):
-        raise FilterArgumentException('the replace filter requires '
-                                      'string replacement arguments')
-    elif not isinstance(count, (int, long)):
-        raise FilterArgumentException('the count parameter of the '
-                                      'replace filter requires '
-                                      'an integer')
+        raise FilterArgumentError('the replace filter requires '
+                                  'string replacement arguments')
     if count is None:
         return s.replace(old, new)
+    if not isinstance(count, (int, long)):
+        raise FilterArgumentError('the count parameter of the '
+                                   'replace filter requires '
+                                   'an integer')
     return s.replace(old, new, count)
 do_replace = stringfilter(do_replace)
 
@@ -96,7 +106,7 @@ def do_lower(s):
 do_lower = stringfilter(do_lower)
 
 
-def do_escape(s, attribute=False):
+def do_escape(attribute=False):
     """
     XML escape ``&``, ``<``, and ``>`` in a string of data. If the
     optional parameter is `true` this filter will also convert
@@ -105,8 +115,65 @@ def do_escape(s, attribute=False):
 
     This method will have no effect it the value is already escaped.
     """
-    return escape(s, attribute)
-do_escape = stringfilter(do_escape)
+    #: because filters are cached we can make a local alias to
+    #: speed things up a bit
+    e = escape
+    def wrapped(env, context, s):
+        if isinstance(s, TemplateData):
+            return s
+        elif hasattr(s, '__html__'):
+            return s.__html__()
+        #: small speedup, do not convert to unicode if we already
+        #: have an unicode object.
+        if s.__class__ is not unicode:
+            s = env.to_unicode(s)
+        return e(s, attribute)
+    return wrapped
+
+
+def do_xmlattr(autospace=False):
+    """
+    Create an SGML/XML attribute string based on the items in a dict.
+    All values that are neither `none` nor `undefined` are automatically
+    escaped:
+
+    .. sourcecode:: html+jinja
+
+        <ul{{ {'class': 'my_list', 'missing': None,
+                'id': 'list-%d'|format(variable)}|xmlattr }}>
+        ...
+        </ul>
+
+    Results in something like this:
+
+    .. sourcecode:: html
+
+        <ul class="my_list" id="list-42">
+        ...
+        </ul>
+
+    As you can see it automatically prepends a space in front of the item
+    if the filter returned something. You can disable this by passing
+    `false` as only argument to the filter.
+
+    *New in Jinja 1.1*
+    """
+    e = escape
+    def wrapped(env, context, d):
+        if not hasattr(d, 'iteritems'):
+            raise TypeError('a dict is required')
+        result = []
+        for key, value in d.iteritems():
+            if value not in (None, env.undefined_singleton):
+                result.append(u'%s="%s"' % (
+                    e(env.to_unicode(key)),
+                    e(env.to_unicode(value), True)
+                ))
+        rv = u' '.join(result)
+        if autospace:
+            rv = ' ' + rv
+        return rv
+    return wrapped
 
 
 def do_capitalize(s):
@@ -186,7 +253,7 @@ def do_default(default_value=u'', boolean=False):
         {{ ''|default('the string was empty', true) }}
     """
     def wrapped(env, context, value):
-        if (boolean and not value) or value in (Undefined, None):
+        if (boolean and not value) or value in (env.undefined_singleton, None):
             return default_value
         return value
     return wrapped
@@ -264,7 +331,7 @@ def do_first():
         try:
             return iter(seq).next()
         except StopIteration:
-            return Undefined
+            return env.undefined_singleton
     return wrapped
 
 
@@ -274,9 +341,9 @@ def do_last():
     """
     def wrapped(env, context, seq):
         try:
-            return iter(_reversed(seq)).next()
-        except (TypeError, StopIteration):
-            return Undefined
+            return iter(reversed(seq)).next()
+        except StopIteration:
+            return env.undefined_singleton
     return wrapped
 
 
@@ -287,8 +354,8 @@ def do_random():
     def wrapped(env, context, seq):
         try:
             return choice(seq)
-        except:
-            return Undefined
+        except IndexError:
+            return env.undefined_singleton
     return wrapped
 
 
@@ -308,10 +375,12 @@ def do_urlencode():
         if isinstance(value, dict):
             tmp = {}
             for key, value in value.iteritems():
-                tmp[env.to_unicode(key)] = env.to_unicode(value)
+                key = env.to_unicode(key).encode(env.charset)
+                value = env.to_unicode(value).encode(env.charset)
+                tmp[key] = value
             return urlencode(tmp)
         else:
-            return quote(env.to_unicode(value))
+            return quote(env.to_unicode(value).encode(env.charset))
     return wrapped
 
 
@@ -354,13 +423,15 @@ def do_filesizeformat():
     return wrapped
 
 
-def do_pprint():
+def do_pprint(verbose=False):
     """
     Pretty print a variable. Useful for debugging.
+
+    With Jinja 1.2 onwards you can pass it a parameter.  If this parameter
+    is truthy the output will be more verbose (this requires `pretty`)
     """
     def wrapped(env, context, value):
-        from pprint import pformat
-        return pformat(value)
+        return pformat(value, verbose=verbose)
     return wrapped
 
 
@@ -473,7 +544,7 @@ def do_textile(s):
     .. _PyTextile: http://dealmeida.net/projects/textile/
     """
     from textile import textile
-    return textile(s)
+    return textile(s.encode('utf-8')).decode('utf-8')
 do_textile = stringfilter(do_textile)
 
 
@@ -486,7 +557,7 @@ def do_markdown(s):
     .. _Python-markdown: http://www.freewisdom.org/projects/python-markdown/
     """
     from markdown import markdown
-    return markdown(s)
+    return markdown(s.encode('utf-8')).decode('utf-8')
 do_markdown = stringfilter(do_markdown)
 
 
@@ -497,14 +568,11 @@ def do_rst(s):
 
     requires `docutils`_.
 
-    .. _docutils: from http://docutils.sourceforge.net/
+    .. _docutils: http://docutils.sourceforge.net/
     """
-    try:
-        from docutils.core import publish_parts
-        parts = publish_parts(source=s, writer_name='html4css1')
-        return parts['fragment']
-    except:
-        return s
+    from docutils.core import publish_parts
+    parts = publish_parts(source=s, writer_name='html4css1')
+    return parts['fragment']
 do_rst = stringfilter(do_rst)
 
 
@@ -556,13 +624,34 @@ def do_format(*args):
             -> Hello? - Foo!
 
     Note that you cannot use the mapping syntax (``%(name)s``)
-    like in python.
+    like in python. Use `|dformat` for that.
     """
     def wrapped(env, context, value):
         return env.to_unicode(value) % args
     return wrapped
 
 
+def do_dformat(d):
+    """
+    Apply python mapping string formatting on an object:
+
+    .. sourcecode:: jinja
+
+        {{ "Hello %(username)s!"|dformat({'username': 'John Doe'}) }}
+            -> Hello John Doe!
+
+    This is useful when adding variables to translateable
+    string expressions.
+
+    *New in Jinja 1.1*
+    """
+    if not isinstance(d, dict):
+        raise FilterArgumentError('dict required')
+    def wrapped(env, context, value):
+        return env.to_unicode(value) % d
+    return wrapped
+
+
 def do_trim(value):
     """
     Strip leading and trailing whitespace.
@@ -594,22 +683,263 @@ def do_capture(name='captured', clean=False):
         {% endfilter %}
         {{ foo }}
     """
-    if not isinstance(name, unicode):
+    if not isinstance(name, basestring):
         raise FilterArgumentError('You can only capture into variables')
     def wrapped(env, context, value):
         context[name] = value
         if clean:
-            return Undefined
+            return TemplateData()
         return value
     return wrapped
 
 
+def do_striptags(value):
+    """
+    Strip SGML/XML tags and replace adjacent whitespace by one space.
+
+    *new in Jinja 1.1*
+    """
+    return ' '.join(_striptags_re.sub('', value).split())
+do_striptags = stringfilter(do_striptags)
+
+
+def do_slice(slices, fill_with=None):
+    """
+    Slice an iterator and return a list of lists containing
+    those items. Useful if you want to create a div containing
+    three div tags that represent columns:
+
+    .. sourcecode:: html+jinja
+
+        <div class="columwrapper">
+          {%- for column in items|slice(3) %}
+            <ul class="column-{{ loop.index }}">
+            {%- for item in column %}
+              <li>{{ item }}</li>
+            {%- endfor %}
+            </ul>
+          {%- endfor %}
+        </div>
+
+    If you pass it a second argument it's used to fill missing
+    values on the last iteration.
+
+    *new in Jinja 1.1*
+    """
+    def wrapped(env, context, value):
+        result = []
+        seq = list(value)
+        length = len(seq)
+        items_per_slice = length // slices
+        slices_with_extra = length % slices
+        offset = 0
+        for slice_number in xrange(slices):
+            start = offset + slice_number * items_per_slice
+            if slice_number < slices_with_extra:
+                offset += 1
+            end = offset + (slice_number + 1) * items_per_slice
+            tmp = seq[start:end]
+            if fill_with is not None and slice_number >= slices_with_extra:
+                tmp.append(fill_with)
+            result.append(tmp)
+        return result
+    return wrapped
+
+
+def do_batch(linecount, fill_with=None):
+    """
+    A filter that batches items. It works pretty much like `slice`
+    just the other way round. It returns a list of lists with the
+    given number of items. If you provide a second parameter this
+    is used to fill missing items. See this example:
+
+    .. sourcecode:: html+jinja
+
+        <table>
+        {%- for row in items|batch(3, '&nbsp;') %}
+          <tr>
+          {%- for column in row %}
+            <tr>{{ column }}</td>
+          {%- endfor %}
+          </tr>
+        {%- endfor %}
+        </table>
+
+    *new in Jinja 1.1*
+    """
+    def wrapped(env, context, value):
+        result = []
+        tmp = []
+        for item in value:
+            if len(tmp) == linecount:
+                result.append(tmp)
+                tmp = []
+            tmp.append(item)
+        if tmp:
+            if fill_with is not None and len(tmp) < linecount:
+                tmp += [fill_with] * (linecount - len(tmp))
+            result.append(tmp)
+        return result
+    return wrapped
+
+
+def do_sum():
+    """
+    Sum up the given sequence of numbers.
+
+    *new in Jinja 1.1*
+    """
+    def wrapped(env, context, value):
+        return sum(value)
+    return wrapped
+
+
+def do_abs():
+    """
+    Return the absolute value of a number.
+
+    *new in Jinja 1.1*
+    """
+    def wrapped(env, context, value):
+        return abs(value)
+    return wrapped
+
+
+def do_round(precision=0, method='common'):
+    """
+    Round the number to a given precision. The first
+    parameter specifies the precision (default is ``0``), the
+    second the rounding method:
+
+    - ``'common'`` rounds either up or down
+    - ``'ceil'`` always rounds up
+    - ``'floor'`` always rounds down
+
+    If you don't specify a method ``'common'`` is used.
+
+    .. sourcecode:: jinja
+
+        {{ 42.55|round }}
+            -> 43
+        {{ 42.55|round(1, 'floor') }}
+            -> 42.5
+
+    *new in Jinja 1.1*
+    """
+    if not method in ('common', 'ceil', 'floor'):
+        raise FilterArgumentError('method must be common, ceil or floor')
+    if precision < 0:
+        raise FilterArgumentError('precision must be a postive integer '
+                                  'or zero.')
+    def wrapped(env, context, value):
+        if method == 'common':
+            return round(value, precision)
+        import math
+        func = getattr(math, method)
+        if precision:
+            return func(value * 10 * precision) / (10 * precision)
+        else:
+            return func(value)
+    return wrapped
+
+
+def do_sort(reverse=False):
+    """
+    Sort a sequence. Per default it sorts ascending, if you pass it
+    `True` as first argument it will reverse the sorting.
+
+    *new in Jinja 1.1*
+    """
+    def wrapped(env, context, value):
+        return sorted(value, reverse=reverse)
+    return wrapped
+
+
+def do_groupby(attribute):
+    """
+    Group a sequence of objects by a common attribute.
+
+    If you for example have a list of dicts or objects that represent persons
+    with `gender`, `first_name` and `last_name` attributes and you want to
+    group all users by genders you can do something like the following
+    snippet:
+
+    .. sourcecode:: html+jinja
+
+        <ul>
+        {% for group in persons|groupby('gender') %}
+            <li>{{ group.grouper }}<ul>
+            {% for person in group.list %}
+                <li>{{ person.first_name }} {{ person.last_name }}</li>
+            {% endfor %}</ul></li>
+        {% endfor %}
+        </ul>
+
+    As you can see the item we're grouping by is stored in the `grouper`
+    attribute and the `list` contains all the objects that have this grouper
+    in common.
+
+    *New in Jinja 1.2*
+    """
+    def wrapped(env, context, value):
+        expr = lambda x: env.get_attribute(x, attribute)
+        return sorted([{
+            'grouper':  a,
+            'list':     list(b)
+        } for a, b in groupby(sorted(value, key=expr), expr)],
+            key=itemgetter('grouper'))
+    return wrapped
+
+
+def do_getattribute(attribute):
+    """
+    Get one attribute from an object. Normally you don't have to use this
+    filter because the attribute and subscript expressions try to either
+    get an attribute of an object or an item. In some situations it could
+    be that there is an item *and* an attribute with the same name. In that
+    situation only the item is returned, never the attribute.
+
+    .. sourcecode:: jinja
+
+        {{ foo.bar }} -> {{ foo|getattribute('bar') }}
+
+    *New in Jinja 1.2*
+    """
+    def wrapped(env, context, value):
+        try:
+            return get_attribute(value, attribute)
+        except (SecurityException, AttributeError):
+            return env.undefined_singleton
+    return wrapped
+
+
+def do_getitem(key):
+    """
+    This filter basically works like the normal subscript expression but
+    it doesn't fall back to attribute lookup. If an item does not exist for
+    an object undefined is returned.
+
+    .. sourcecode:: jinja
+
+        {{ foo.bar }} -> {{ foo|getitem('bar') }}
+
+    *New in Jinja 1.2*
+    """
+    def wrapped(env, context, value):
+        try:
+            return value[key]
+        except (TypeError, KeyError, IndexError, AttributeError):
+            return env.undefined_singleton
+    return wrapped
+
+
 FILTERS = {
     'replace':              do_replace,
     'upper':                do_upper,
     'lower':                do_lower,
     'escape':               do_escape,
     'e':                    do_escape,
+    'xmlattr':              do_xmlattr,
     'capitalize':           do_capitalize,
     'title':                do_title,
     'default':              do_default,
@@ -640,6 +970,17 @@ FILTERS = {
     'string':               do_string,
     'urlize':               do_urlize,
     'format':               do_format,
+    'dformat':              do_dformat,
     'capture':              do_capture,
-    'trim':                 do_trim
+    'trim':                 do_trim,
+    'striptags':            do_striptags,
+    'slice':                do_slice,
+    'batch':                do_batch,
+    'sum':                  do_sum,
+    'abs':                  do_abs,
+    'round':                do_round,
+    'sort':                 do_sort,
+    'groupby':              do_groupby,
+    'getattribute':         do_getattribute,
+    'getitem':              do_getitem
 }
